//=========================================== Support_Cloak =======================================================================================================\\
// Useful Cloak player variables                                                                                                                                   \\
//-Variable Name----------Type---------Function--------------------------------------------------------------------------------------------------------------------\\
// isCloaked              Boolean      Tells if the player is currently fully cloaked.                                                                             \\
// isCloaking             Boolean      Tells if the player is currently cloaking.                                                                                  \\
// isUnCloaking           Boolean      Tells if the player is currently uncloaking.                                                                                \\
// noCloak                Boolean      Tells if the player is allowed to cloak.                                                                                    \\
// cloakTime			  Integer      Time it takes to cloak/uncloak, mostly for automatic cloak/uncloak functions.                                               \\
// cloakAllowToolEquip    Boolean      Determines if the player may equip tools while cloaked.                                                                     \\
// cloakAllowToolUse      Boolean      Determines if the player may use tools while cloaked.                                                                       \\
// cloakAllowEmote        Boolean      Determines if the player may emote while cloaked.                                                                           \\
// cloakAllowCapture      Boolean      Determines if the player may capture while cloaked.                                                                         \\
// cloakAllowJump         Boolean      Determines if the player may jump while cloaked.                                                                            \\
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------\\
// Useful Cloak functions                                                                                                                                          \\
//-Function Name----------Args----------------Function-------------------------------------------------------------------------------------------------------------\\
// Player::cloakStart(    %player, %time)     Begins to cloak the given %player, but only if %player.noCloak != true over the given %time in ms.                   \\
// Player::cloakEnd(      %player, %time)	  Ends the cloak of the given %player over the given %time in ms.                                                      \\
//    Note: The above functions don't use the cloakTime player variable because you can have more control over (un)cloaking this way. :D                           \\
// Player::cloakDisrupt(  %player, %time)     Disrupts the cloak of the given %player, making him visible for the given %time in miliseconds.                      \\
// Player::toggleCloak(   %player, %time)	  Toggles the cloak of the given %player over the given %time in ms.                                                   \\
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------\\
// Datablock specific functions                                                                                                                                    \\
//-Function Name-------------------Args------Function--------------------------------------------------------------------------------------------------------------\\
// DatablockName::onCloakStart(   %player)   Function that gets called (if it exists) before the cloak starts (return 1 to cancel the cloak if needed).            \\
// DatablockName::onCloakStarted( %player)   Function that gets called (if it exists) after the cloak starts (use %player.cloakEnd to stop the cloak if needed).   \\
// DatablockName::onCloakEnd(     %player)   Function that gets called (if it exists) before the cloak ends (return 1 to cancel ending the cloak if needed).       \\
// DatablockName::onCloakEnded(   %player)   Function that gets called (if it exists) after the cloak ends (use %player.cloakStart to restart the cloak if needed).\\
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------\\
//                                                                                                                                                                 \\
// If you have suggestions/bugs related to this  script, please report them to me.                                                                                 \\
// You can contact me via the following:                                                                                                                           \\
// Forum name: Lordician                                                                                                                                           \\
// Blockland name: [MAS]Lordician  BL_ID: 20598                                                                                                                    \\
// Steam name: Lordician                                                                                                                                           \\
//                                                                                                                                                                 \\
//---------------------------------------------- Credits ----------------------------------------------------------------------------------------------------------\\
// Space Guy, for making the original script of which i based this on. (~90% is from the original script)                                                        \\
//=================================================================================================================================================================\\

if($Server::cloakSupportVersion > 2.1)
   return;

$Server::cloakSupportVersion = 2.1;

//Cloaking functions//
//Start cloaking
function Player::cloakStart (%player, %time) {

	%data = %player.getDatablock();
	if (isFunction(%data.getName(), "onCloakStart")) {
		
		%cancel = 0;
		eval("%cancel = " @ %data.getname() @ "::onCloakStart(%player);");
		
		if (%cancel) { return; }
		
	}
	
	if (%player.noCloak) { return; }
	
	//serverplay3d(CloakStartSound,%player.getPosition());
	
	if (%player.isCloaked || %player.isCloaking) { return; }

	if (%player.getState() $= "Dead") { return; }
	
	if (%time >= 10)
	{
		%player.isCloaking = 1;
		%player.doCloakAnimation(1, 0, (%time/10));
	}
	else
	{
		%player.doCloakAnimation(1, 1, 10); //Instant done cloak animation :D
	}
	%player.isUncloaking = 0;
}

//End cloaking
function Player::cloakEnd (%player, %time) {
	
	%data = %player.getDatablock();
	if (isFunction(%data.getName(), "onCloakEnd")) {
		
		%cancel = 0;
		eval("%cancel = " @ %data.getname() @ "::onCloakEnd(%player);");
		
		if (%cancel) { return; }
		
	}
	
	if (!%player.isCloaked || %player.isCloaking) { return; }

	if (%player.getState() $= "Dead") { return; }
	
	if (isEventPending(%player.cloakAnimSched)) { cancel(%player.cloakAnimSched); } //Stops cloaking animation (Stops disrupt from glitching out and ending up in unregistered cloak)
	
	//serverplay3d(CloakEndSound,%player.getPosition());
	%player.unHideNode("ALL");

	if (isObject(%player.client)) {
		
		%player.client.applyBodyParts();
		%player.client.applyBodyColors();
		
	} else {
		
		applyDefaultCharacterPrefs(%player);
		
	}

	%player.setNodeColor("ALL","0 0 0 0");
	if (%time >= 10)
	{
		%player.isUncloaking = 1;
		%player.doCloakAnimation(0, 0, (%time/10));
	}
	else
	{
		%player.doCloakAnimation(0, 1, 10); //Instant done uncloak animation :D
	}
	%player.isCloaked = 0;
	
	if (isEventPending(%player.cloakSched)) { cancel(%player.cloakSched); }
}

//Disrupt cloaking
function Player::cloakDisrupt (%player, %time) {
	
	if (%player.isCloaking || %player.isUncloaking) { return; } //Cannot disrupt cloaking... :/
	
	//serverplay3d(CloakDisruptSound,%player.getPosition());
	if (isObject(%player.client)) {
		
		if (isObject(%player.client.minigame) && %player.client.tdmTeam != -1 && %player.client.tdmTeam !$= "") {
			
			%col = getColorIDTable(%player.client.minigame.teamCol[%player.client.tdmTeam]);
			
		} else {
			
			%col = %player.client.chestColor;
			
		}
		
	} else {
		
		%col = "1 1 1 1";
		
	}
	
	%player.unHideNode("ALL");
	if (isObject(%player.client)) {
		
		%player.client.applyBodyParts();
		%player.client.applyBodyColors();
		
	} else {
		
		applyDefaultCharacterPrefs(%player);
		
	}
		
	%player.setNodeColor("ALL", getWords(%col, 0, 2) SPC 0.5);
	if (%time >= 0) { %player.cloakAnimSched = %player.schedule(%time, hideNode, "ALL"); }
}

//Toggles cloaking
function Player::toggleCloak (%player, %time) {
	
	if (%player.isCloaked) {
		
		%player.cloakEnd(%time);
		
	} else if (!%player.noCloak) {
		
		%player.cloakStart(%time);
		
	}
	
}

//Cloak animation function
function Player::doCloakAnimation (%player,%direction,%amt) {
	
	if (%player.getState() $= "Dead") {
		
		cancel(%player.cloakAnimSched);
		return;
		
	}
	
	if (%direction == 1 && %player.noCloak) {
		
		%direction = 0;
		%amt = 1 - %amt;
		
	}
   
	if (isObject(%player.client)) {
		
		if (isObject(%player.client.minigame) && %player.client.tdmTeam != -1 && %player.client.tdmTeam !$= "") {
			
			%col = getColorIDTable(%player.client.minigame.teamCol[%player.client.tdmTeam]);
			
		} else {
			
			%col = %player.client.chestColor;
			
		}
		
	} else {
		
		%col = "1 1 1 1";
		
	}

	%amt+=0.1;
	if (%direction == 1) {
		
		%player.setNodeColor("ALL", getWords(%col, 0, 2) SPC (1-%amt) / 2);
		
		if (%amt >= 1) {
			
			if (!%player.cloakAllowToolEquip) { %player.unMountImage(0); } //Unmounts the current tool if the kind of cloak does not allow tools to be equipped during cloaking.
			
			%player.isCloaking = 0;
			%player.isCloaked = 1;
			%player.hideNode("ALL");
			%player.setShapeNameDistance(0);
			
			%data = %player.getDatablock();
			if (isFunction(%data.getName(), "onCloakStarted")) {
				
				eval(%data.getname() @ "::onCloakStarted(%player);");
				
			}
			
			return;
			
		}
		
	} else {
		
		%player.setNodeColor("ALL",getWords(%col,0,2) SPC %amt/2);
		if (%amt >= 1) {
			
			%player.isCloaking = 0;
			%player.isCloaked = 0;
			%player.lastUnCloakTime = getSimTime();
 
			if (isObject(%player.client)) {
				
				%player.client.applyBodyColors();
				
			} else {
				
				applyDefaultCharacterPrefs(%player);
				
			}
			
			

			if(fileName(%player.dataBlock.shapeFile) $= "m.dts")
				%player.unHideNode("headSkin");
			
			%mini = getMinigameFromObject(%player);
			if (%mini.nameDistance !$= "") {
				
				%player.setShapeNameDistance(%mini.nameDistance);
				
			} else { %player.setShapeNameDistance(600); }
			
			%data = %player.getDatablock();
			if (isFunction(%data.getName(), "onCloakEnded")) {
				
				eval(%data.getname() @ "::onCloakEnded(%player);");
				
			}
			
			return;
		}
	}
	%player.cloakAnimSched = %player.schedule(100,doCloakAnimation,%direction,%amt);
}

//Package stuff
if (isPackage(CloakSupportCode)) {
	
	deactivatePackage(CloakSupportCode);	//De-activates package with the same name if there is any. Safety measure.
	
}

package CloakSupportCode {
	
	function Armor::onNewDatablock (%this,%player) {
		Parent::onNewDatablock(%this,%player);
		
		//Any datablock reading can go here if i can get on any that goes when the player gets a new datablock...
		
		//if (%this.canCloak)
		//{
			//%player.allowCloak = 1;
		//}
		
	}
	
	function Armor::onDisabled (%this,%player) {
		if (%player.isCloaked || %player.isCloaking) {
			
			%player.unHideNode("ALL");
			
			if (isObject(%player.client)) {
				
				%player.client.applyBodyParts();
				%player.client.applyBodyColors();
				
			} else {
				
				applyDefaultCharacterPrefs(%player);
				
			}
			
		}
		
		cancel(%player.cloakAnimSched);
		cancel(%player.cloakSched);
		
		Parent::onDisabled(%this,%player);
		
	}
	
	function Armor::damage (%data, %player, %sourceObject, %position, %damage, %damagetype) {
		
		if (%player.getState() !$= "Dead" && %player.isCloaked && !%player.isCloaking && %player.cloakDamageDisrupt) {
			
			%player.cloakDisrupt(300);
			
		}
		
		Parent::damage(%data, %player, %sourceObject, %position, %damage, %damageType);
		
	}
	
	function Armor::onTrigger (%data,%player,%slot,%io) {
		
		if (%slot == 0 && !%player.cloakAllowToolUse && !%player.getMountedImage(0).stealthWeapon) {
			
			%player.noCloak = %io;
			
			if (%io && %player.isCloaked) {
				
				//A shot has been fired during cloak, but the current kind of cloak doesn't allow that, so we end the cloak.
				%player.cloakEnd(%player.cloakTime);
				
			}
			
		}
		
		if (%slot == 2 && %player.isCloaked && !%player.cloakAllowJump) {
			
			%player.noCloak = %io;
			
			if (%io) {
				
				//NO JUMPING ALLOWED!
				%player.cloakEnd(%player.cloakTime);
				
			}
			
		}
		
		Parent::onTrigger(%data, %player, %slot, %io);
		
	}
	
	function Player::ActivateStuff(%obj) {
		
		if (%obj.isCloaked && !%obj.cloakAllowToolUse) {
			
			%obj.cloakEnd(%player.cloakTime);
			
		}
		
		Parent::ActivateStuff(%obj);
	}
	
	function ctfFlagItem::onPickup (%this, %obj, %player, %amt) {
		
		if ((%player.isCloaked || %player.isCloaking) && !%player.cloakAllowCapture) {
			
			if (isObject(%player.client)) {
				
				%player.client.centerPrint("\c5You cannot pick up the \c3Flag\c5 while \c3Cloaked\c5!",2);
				
			}
			
			return;
			
		}
		
		Parent::onPickup(%this, %obj, %player, %amt);
		
	}
	
	function Player::clearTempColor(%this) {
		
		if (%this.isCloaked || %this.isCloaking) {
			
			return;
			
		}

		Parent::clearTempColor(%this);
		
	}
   
	function Player::emote (%this, %type, %skipSpamProtect) {
		
		if (%this.isCloaked && !%player.cloakAllowEmote) { return; }

		Parent::emote(%this,%type,%skipSpamProtect);
		
	}
	
	function ShapeBase::setShapeNameDistance (%this, %d) {
		
		//minimum possible
		if (%this.getType() & $TypeMasks::PlayerObjectType && %this.isCloaked) { %d = 0; }
		
		Parent::setShapeNameDistance(%this, %d);
		
	}
	
	
	//Equipping/Force-Equipping an item ends the cloak if Tool Equipping is not allowed with the current cloak of the player, so we do all that here, spam functions inbound!
	function servercmduseSprayCan (%client, %slot) {
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip) && (!isObject(%client.minigame) || %client.minigame.enablePainting)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmduseSprayCan(%client,%slot);
	}

	function servercmduseFXCan (%client, %slot) {
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip) && (!isObject(%client.minigame) || %client.minigame.enablePainting)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmduseFXCan(%client,%slot);
	}
 
	function servercmduseinventory (%client, %slot) { //use bricks 
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmduseinventory(%client, %slot);
	}
 
	function servercmdinstantusebrick (%client, %a, %b, %c, %d, %e) { //use bricks 
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmdinstantusebrick(%client, %a, %b, %c, %d, %e);
	}
 
	function servercmdmagicwand (%client) {
		if ((%client.isAdmin || %client.isSuperAdmin) && ((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmdmagicwand(%client);
	}

	function servercmdwand (%client) {
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip) && (%client.minigame.enableWand || !isObject(%client.minigame))) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmdwand(%client);
	}
 
	function servercmdusehammer (%client) {
		if ((%client.isAdmin || %client.isSuperAdmin) && ((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmdusehammer(%client);
	}
 
	function servercmduseprintgun (%client) {
		if ((%client.isAdmin || %client.isSuperAdmin) && ((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip)) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::servercmduseprintgun(%client);
	}
	
	//For any other force-mounted image, just disable it e.g. Duplicator, Fill Can, Multi Tool
	function ShapeBase::mountImage (%this, %im, %slot, %loaded, %skinTag) {
		if (((%client.player.isCloaked || %client.player.isCloaking) && !%client.player.cloakAllowToolEquip) && %slot == 0) {
			%client.player.cloakEnd(%client.player, %client.player.cloakTime);
		}
		Parent::mountImage(%this, %im, %slot, %loaded, %skinTag);
	}
	
};
activatePackage(CloakSupportCode);